"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "BaseGenerator", {
    enumerable: true,
    get: function() {
        return BaseGenerator;
    }
});
const _colors = /*#__PURE__*/ _interop_require_default(require("colors"));
const _path = /*#__PURE__*/ _interop_require_wildcard(require("path"));
const _updatenotifier = /*#__PURE__*/ _interop_require_wildcard(require("update-notifier"));
const _yeomangenerator = /*#__PURE__*/ _interop_require_default(require("yeoman-generator"));
const _nodecorelibrary = require("@rushstack/node-core-library");
const _utilities = /*#__PURE__*/ _interop_require_wildcard(require("./utilities"));
const _PackageJsonManager = require("./PackageJsonManager");
const _ConfigJsonManager = require("./ConfigJsonManager");
const _ServeJsonManager = require("./ServeJsonManager");
const _PackageSolutionJsonManager = require("./PackageSolutionJsonManager");
const _YeomanConfiguration = require("./YeomanConfiguration");
function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interop_require_wildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {
        __proto__: null
    };
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
class BaseGenerator extends _yeomangenerator.default {
    get packageJsonManager() {
        if (!_PackageJsonManager.PackageJsonManager.instance) {
            _PackageJsonManager.PackageJsonManager.load(this.destinationPath('package.json'), this.fs);
        }
        if (!this.allowEmptyPackageJson && !_PackageJsonManager.PackageJsonManager.instance.data) {
            throw new Error(`Cannot find "package.json" file required by this generator`);
        }
        return _PackageJsonManager.PackageJsonManager.instance;
    }
    get configJson() {
        if (!_ConfigJsonManager.ConfigJsonManager.instance) {
            _ConfigJsonManager.ConfigJsonManager.load(this.destinationPath('config/config.json'), this.fs);
        }
        return _ConfigJsonManager.ConfigJsonManager.instance;
    }
    get serveJson() {
        if (!_ServeJsonManager.ServeJsonManager.instance) {
            _ServeJsonManager.ServeJsonManager.load(this.destinationPath('config/serve.json'), this.fs);
        }
        return _ServeJsonManager.ServeJsonManager.instance;
    }
    get packageSolutionJson() {
        if (!_PackageSolutionJsonManager.PackageSolutionJsonManager.instance) {
            _PackageSolutionJsonManager.PackageSolutionJsonManager.load(this.destinationPath('config/package-solution.json'), this.fs);
        }
        return _PackageSolutionJsonManager.PackageSolutionJsonManager.instance;
    }
    static _checkForUpdates() {
        if (!BaseGenerator._hasCheckedForUpdates) {
            const update = new _updatenotifier.UpdateNotifier({
                pkg: BaseGenerator.generatorPackageJson
            });
            update.notify({
                defer: false
            });
        }
        BaseGenerator._hasCheckedForUpdates = true;
    }
    /** Where you write the generator specific files (routes, controllers, etc) */ writing(shouldCopy = true) {
        // Update config to add the webpart entry.
        this.configJson.save();
        // Update package.json dependencies and trigger npm install if necessary.
        this.packageJsonManager.save();
        // Update serve.json to add the new serve configuration.
        this.serveJson.save();
        // Update package-solution.json to add the new packaging configuration.
        this.packageSolutionJson.save();
        if (shouldCopy) {
            this.copyTemplate(this.sourceRoot(), this.destinationRoot());
        }
    }
    tryInstall() {
        const shouldInstall = !this.options['skip-install'];
        if (shouldInstall && _YeomanConfiguration.YeomanConfiguration.packageManager === 'pnpm') {
            this._doPnpmInstall();
        }
        return shouldInstall;
    }
    ensureCorrectFolder() {
        if (this.config.get('whichFolder') === 'subdir') {
            this.destinationRoot(this.destinationPath());
        }
    }
    ensureDependencyGroup(group) {
        const dependenciesByDependencyGroup = this._ensureDependenciesByDependencyGroup();
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const dependencies = dependenciesByDependencyGroup.get(group);
        for (const [name, version] of Object.entries(dependencies.dependencies)){
            this.packageJsonManager.addDependency(name, version);
        }
        // before applying devDependencies, dedupe them
        this._dedupeDependencies(dependencies.devDependencies);
        for (const [name, version] of Object.entries(dependencies.devDependencies)){
            this.packageJsonManager.addDevDependency(name, version);
        }
        // if at any point we are adding some react typings, then ensure they are in the resolutions for Yarn
        this._addResolutionsIfApplicable([
            '@types/react'
        ]);
    }
    nameIsDependencyGroup(name) {
        const dependenciesByDependencyGroup = this._ensureDependenciesByDependencyGroup();
        return dependenciesByDependencyGroup.has(name);
    }
    _ensureDependenciesByDependencyGroup() {
        if (!this._dependenciesByDependencyGroup) {
            const dependenciesFile = _nodecorelibrary.JsonFile.load(`${__dirname}/dependencies.json`);
            const plusBeta = this.options.plusbeta;
            const dependenciesByDependencyGroup = new Map();
            this._dependenciesByDependencyGroup = dependenciesByDependencyGroup;
            for (const [groupName, groupDependencies] of Object.entries(dependenciesFile)){
                const dependencyList = {
                    devDependencies: {},
                    ...plusBeta && groupDependencies.plusBeta ? groupDependencies.plusBeta : groupDependencies.standard
                };
                dependenciesByDependencyGroup.set(groupName, dependencyList);
            }
        }
        return this._dependenciesByDependencyGroup;
    }
    _dedupeDependencies(devDependencies) {
        // if there is already a dependency and the same devDependency - leave dependency only
        if (!this.packageJsonManager.data.dependencies || !devDependencies) {
            return;
        }
        const { dependencies } = this.packageJsonManager.data;
        Object.keys(devDependencies).forEach((packageName)=>{
            if (dependencies[packageName]) {
                delete devDependencies[packageName];
            }
        });
    }
    _addResolutionsIfApplicable(packages) {
        if (this.packageJsonManager.data.dependencies) {
            for (const packageName of packages){
                let packageVersion;
                if (packageVersion = this.packageJsonManager.data.dependencies[packageName]) {
                    this.packageJsonManager.addResolution(packageName, packageVersion);
                }
            }
        }
    }
    _doPnpmInstall() {
        // Awaiting: https://github.com/yeoman/generator/pull/1067
        const installer = 'pnpm';
        const args = [
            'install'
        ];
        // Follow the same pattern as runInstall(), which uses "once" to ensure only
        // one install for both the solution and the component generators.
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        this.env.runLoop.add('install', (done)=>{
            this.emit(installer + 'Install');
            this.spawnCommand(installer, args).on('error', (err)=>{
                // eslint-disable-next-line no-console
                console.log(_colors.default.red('Could not finish installation. \n') + 'Please install ' + installer + ' with ' + _colors.default.yellow('npm install -g ' + installer) + ' and try again.');
            }).on('exit', (err)=>{
                this.emit(installer + 'Install:end');
                done();
            });
        }, {
            once: installer + ' ' + args.join(' '),
            run: false
        });
    }
    _untokenizedCopier(directory, destination) {
        return (filename)=>{
            const sourceFile = _path.join(directory, filename);
            const destinationFile = _path.join(destination, _utilities.untokenize(filename, this.context));
            this.fs.copyTpl(sourceFile, destinationFile, this.context);
        };
    }
    constructor(args, options){
        // Always overwrite the conflict files, without any prompting.
        super(args, !(options.force = true) || options);
        /**
   * The context property bag used by templates
   */ _define_property(this, "context", void 0);
        _define_property(this, "allowEmptyPackageJson", false);
        _define_property(this, "_dependenciesByDependencyGroup", void 0);
        _define_property(this, "copyTemplate", (sourceDirectory, destinationDir)=>{
            _utilities.filesIn(sourceDirectory).forEach(this._untokenizedCopier(sourceDirectory, destinationDir));
        });
        this.context = {};
        // Register an error handler for uncaught exceptions:
        this.addListener('error', (errorObject)=>{
            let errorMessage = '';
            if (errorObject) {
                if (typeof errorObject === 'string') {
                    errorMessage = errorObject || '';
                } else if (errorObject instanceof Error) {
                    errorMessage = errorObject.message || '';
                }
            }
            if (!errorMessage.trim()) {
                errorMessage = 'The operation was unsuccessful';
            }
            // eslint-disable-next-line no-console
            console.error('\n' + _colors.default.red('Error: ' + errorMessage));
            process.exitCode = 1;
        });
        _YeomanConfiguration.YeomanConfiguration.setStore(this.config);
        BaseGenerator._checkForUpdates();
        this.option('skip-install', {
            type: Boolean,
            description: 'Skip running package managers (NPM, etc) post scaffolding',
            default: false
        });
    }
}
_define_property(BaseGenerator, "generatorPackageJson", _nodecorelibrary.JsonFile.load(`${__dirname}/../../package.json`));
_define_property(BaseGenerator, "_hasCheckedForUpdates", false);

//#sourceMappingUrl=./BaseGenerator.js.map