"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "PackageSolutionJsonManager", {
    enumerable: true,
    get: function() {
        return PackageSolutionJsonManager;
    }
});
const _YeomanConfiguration = require("./YeomanConfiguration");
const _JsonManager = require("./JsonManager");
function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
class PackageSolutionJsonManager extends _JsonManager.JsonManager {
    static load(filepath, fs) {
        if (this._instance) {
            throw new Error(`Cannot create a second instance of PackageSolutionJsonManager`);
        }
        let data;
        try {
            data = fs.readJSON(filepath);
        } catch (error) {
            data = undefined;
        }
        if (!data) {
            data = {
                $schema: 'https://developer.microsoft.com/json-schemas/spfx-build/package-solution.schema.json'
            };
        }
        PackageSolutionJsonManager._instance = new PackageSolutionJsonManager(filepath, fs, data);
    }
    static reset() {
        PackageSolutionJsonManager._instance = undefined;
    }
    static get instance() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this._instance;
    }
    getSkipFeatureDeployment() {
        try {
            return !!this.data.solution?.skipFeatureDeployment;
        } catch (error) {
            return false;
        }
    }
    setUpSolution(solutionContext) {
        const data = this.data;
        data.solution = {
            name: `${solutionContext.libraryName}-client-side-solution`,
            id: solutionContext.libraryId,
            version: '1.0.0.0'
        };
        data.solution.includeClientSideAssets = true;
        if (solutionContext.skipFeatureDeployment) {
            data.solution.skipFeatureDeployment = solutionContext.skipFeatureDeployment;
        }
        if (solutionContext.isDomainIsolated !== undefined) {
            data.solution.isDomainIsolated = solutionContext.isDomainIsolated;
        }
        if (_YeomanConfiguration.YeomanConfiguration.environment === 'spo') {
            data.solution.developer = {
                name: '',
                websiteUrl: '',
                privacyUrl: '',
                termsOfUseUrl: '',
                mpnId: `Undefined-${_YeomanConfiguration.YeomanConfiguration.version}`
            };
        }
        data.paths = {
            zippedPackage: `solution/${solutionContext.libraryName}.sppkg`
        };
        data.solution.metadata = {
            shortDescription: {
                default: solutionContext.solutionShortDescription
            },
            longDescription: {
                default: solutionContext.solutionShortDescription
            },
            screenshotPaths: [],
            videoUrl: '',
            categories: []
        };
    }
    addExtensionFeature(featureId, includeClientSideInstance) {
        this.addFeature(featureId, 'Application Extension - Deployment of custom action', 'Deploys a custom action with ClientSideComponentId association', true, includeClientSideInstance);
    }
    addFeature(featureId, title, description, includeAssets, includeClientSideInstance = false) {
        const solution = this.data.solution;
        if (!solution) {
            // This should never happened as the solution has been set up
            return;
        }
        let feature = this._tryGetFeatureToReuse(solution);
        const isAddingFeature = !feature;
        if (!feature) {
            // we could use isAddingFeature flag but it leads to 'possibly undefined' error when using feature below
            if (!solution.features) {
                solution.features = [];
            }
            feature = {
                title: title,
                description: description,
                id: featureId,
                version: '1.0.0.0'
            };
        }
        if (includeAssets) {
            if (!feature.assets) {
                feature.assets = {};
            }
            let manifests = feature.assets.elementManifests;
            if (!manifests) {
                manifests = feature.assets.elementManifests = [
                    'elements.xml'
                ];
            } else {
                if (manifests.indexOf('elements.xml') === -1) {
                    manifests.push('elements.xml');
                }
            }
            if (includeClientSideInstance && manifests.indexOf('ClientSideInstance.xml') === -1) {
                manifests.push('ClientSideInstance.xml');
            }
        }
        if (isAddingFeature) {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            solution.features.push(feature);
        }
    }
    /**
   * Returns the existing feature that doesn't have componentIds.
   * We can use the same feature to add additional elements
   */ _tryGetFeatureToReuse(solution) {
        if (!solution.features) {
            return undefined;
        }
        const noComponendIdsFeatures = solution.features.filter((feature)=>!feature.componentIds || !feature.componentIds.length);
        return noComponendIdsFeatures.length ? noComponendIdsFeatures[0] : undefined;
    }
}
_define_property(PackageSolutionJsonManager, "_instance", void 0);

//#sourceMappingUrl=./PackageSolutionJsonManager.js.map