"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    checkSolutionFolder: function() {
        return checkSolutionFolder;
    },
    compose: function() {
        return compose;
    },
    filesIn: function() {
        return filesIn;
    },
    generateGuid: function() {
        return generateGuid;
    },
    titleCase: function() {
        return titleCase;
    },
    untokenize: function() {
        return untokenize;
    }
});
const _colors = /*#__PURE__*/ _interop_require_default(require("colors"));
const _fs = /*#__PURE__*/ _interop_require_wildcard(require("fs"));
const _lodash = /*#__PURE__*/ _interop_require_wildcard(require("lodash"));
const _nodecorelibrary = require("@rushstack/node-core-library");
const _path = /*#__PURE__*/ _interop_require_wildcard(require("path"));
const _uuid = /*#__PURE__*/ _interop_require_wildcard(require("uuid"));
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interop_require_wildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {
        __proto__: null
    };
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
function titleCase(str) {
    str = _lodash.camelCase(str);
    return _lodash.upperFirst(str);
}
function untokenize(str, props) {
    for (const [prop, propValue] of Object.entries(props)){
        if (typeof propValue === 'string') {
            // eslint-disable-next-line @rushstack/security/no-unsafe-regexp
            str = str.replace(new RegExp(`{${prop}}`, 'g'), propValue);
        }
    }
    // Convert file name prefix with '_' to '.'
    const basename = _path.basename(str);
    if (basename[0] === '_') {
        const filename = '.' + basename.substr(1);
        const dirname = _path.dirname(str);
        str = _path.join(dirname, filename);
    }
    return str;
}
function filesIn(...args) {
    const inputPath = _path.join.apply(undefined, args);
    const files = [];
    function traversePath(innerPath) {
        const subPaths = _fs.readdirSync(innerPath).filter((filename)=>filename !== '.' && filename !== '..').map((filename)=>_path.join(innerPath, filename));
        const subFiles = subPaths.filter((pathname)=>_fs.statSync(pathname).isFile()).map((pathname)=>_path.relative(inputPath, pathname));
        const subFolders = subPaths.filter((pathname)=>_fs.statSync(pathname).isDirectory());
        subFolders.forEach(traversePath);
        files.push.apply(files, subFiles);
    }
    traversePath(inputPath);
    return files;
}
async function checkSolutionFolder(generator, solutionName) {
    if (!solutionName) {
        generator.log(_colors.default.red('!!! Warning !!!'));
        const answers = await generator.prompt([
            {
                type: 'confirm',
                name: 'continue',
                default: undefined,
                message: 'You are not running under solution folder. Continue?'
            }
        ]);
        if (answers.continue) {
            return solutionName; // resolve with the solution name
        } else {
            throw generator.log(_colors.default.red('Bye!'));
        }
    } else {
        return solutionName;
    }
}
function generateGuid() {
    return _uuid.v4();
}
function compose(generator, base, options) {
    base.composeWith(_nodecorelibrary.Import.resolveModule({
        baseFolderPath: __dirname,
        modulePath: `@microsoft/generator-sharepoint/lib/generators/${generator}`,
        allowSelfReference: true
    }), options);
}

//#sourceMappingUrl=./utilities.js.map