"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    BaseComponentGenerator: function() {
        return BaseComponentGenerator;
    },
    defineOptions: function() {
        return defineOptions;
    },
    normalizeComponentNames: function() {
        return normalizeComponentNames;
    }
});
const _nodecorelibrary = require("@rushstack/node-core-library");
const _colors = /*#__PURE__*/ _interop_require_default(require("colors"));
const _lodash = /*#__PURE__*/ _interop_require_wildcard(require("lodash"));
const _path = /*#__PURE__*/ _interop_require_wildcard(require("path"));
const _BaseGenerator = require("../../common/BaseGenerator");
const _utilities = /*#__PURE__*/ _interop_require_wildcard(require("../../common/utilities"));
const _YeomanConfiguration = require("../../common/YeomanConfiguration");
function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interop_require_wildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {
        __proto__: null
    };
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
function defineOptions(generator, type) {
    generator.option('component-description', {
        description: `${type} description`,
        type: String
    });
    generator.option('component-name', {
        description: `${type} name`,
        type: String
    });
    generator.option('framework', {
        // NOTE: 'framemowrk' name is kept for backward compatibility
        description: 'Template to use. Available: "none", "react", "minimal"',
        type: String
    });
    generator.option('plusbeta', {
        description: 'Use the beta packages',
        type: Boolean
    });
}
class BaseComponentGenerator extends _BaseGenerator.BaseGenerator {
    async prompting() {
        await this._checkSolutionAsync();
        const availableTemplates = [
            {
                name: 'No framework',
                value: 'none'
            }
        ];
        if (this.allowedTemplates.indexOf('react') !== -1) {
            availableTemplates.push({
                name: 'React',
                value: 'react'
            });
        }
        if (this.allowedTemplates.indexOf('minimal') !== -1) {
            availableTemplates.unshift({
                name: 'Minimal',
                value: 'minimal'
            });
        }
        const answers = await this.prompt([
            {
                type: 'input',
                name: 'componentName',
                default: 'HelloWorld',
                when: ()=>!this.config.get('componentName'),
                message: `What is your ${this.friendlyName} name?`,
                validate: (input)=>{
                    const normalizedNamesForValidation = normalizeComponentNames(input, this.codeName);
                    const outputFolderPathForValidation = this._getOutputFolder(normalizedNamesForValidation.componentNameCamelCase);
                    if (_nodecorelibrary.FileSystem.exists(outputFolderPathForValidation)) {
                        // eslint-disable-next-line no-console
                        console.log(_colors.default.yellow(`\nThe folder "${outputFolderPathForValidation}" already exists.` + ` Please choose a different name for your component.`));
                        return false;
                    }
                    // disallow quotes, since this will mess with the JSON we put this string into
                    if (input.indexOf('"') !== -1) {
                        // eslint-disable-next-line no-console
                        console.log(_colors.default.yellow(`\nDo not use double quotes in your title.`));
                        return false;
                    }
                    return true;
                }
            },
            {
                type: 'list',
                name: 'template',
                when: ()=>!this.config.get('framework') && !this.config.get('template') && availableTemplates.length !== 1,
                message: 'Which template would you like to use?',
                choices: availableTemplates
            }
        ]);
        const componentName = this.config.get('componentName') || answers.componentName;
        this.context.template = this.config.get('framework') || this.config.get('template') || answers.template || 'none';
        this.context.componentDescription = this.config.get('componentDescription') || `${componentName} description`;
        this.context.componentId = _utilities.generateGuid();
        const normalizedNames = normalizeComponentNames(componentName, this.codeName);
        this.context.componentNameUnescaped = normalizedNames.componentNameUnescaped;
        this.context.componentName = normalizedNames.componentName;
        this.context.componentNameCamelCase = normalizedNames.componentNameCamelCase;
        this.context.componentClassName = normalizedNames.componentClassName;
        this.context.componentStrings = normalizedNames.componentStrings;
        this.context.componentClassNameKebabCase = normalizedNames.componentClassNameKebabCase;
        this.context.componentAlias = normalizedNames.componentAlias;
        this.context.eslintProfile = this.context.template === 'react' ? 'react' : 'default';
        // the folder where we will drop the information
        const outputFolderPath = this._getOutputFolder(this.context.componentNameCamelCase);
        if (_nodecorelibrary.FileSystem.exists(outputFolderPath)) {
            throw new Error(`The folder "${outputFolderPath}" already exists.` + ` Please choose a different name for your component.`);
        }
    }
    configuring(bundleEntry) {
        const templateName = this.context.template;
        if (this.nameIsDependencyGroup(templateName)) {
            this.ensureDependencyGroup(templateName);
        }
        if (!this.configJson.data) {
            throw new Error(`Missing required file: "config/config.json"`);
        }
        try {
            if (bundleEntry) {
                this.configJson.addEntry(bundleEntry, this.context.componentClassNameKebabCase);
            } else {
                this.configJson.addEntry({
                    components: [
                        {
                            entrypoint: `./lib/${this.folderName}/${this.context.componentNameCamelCase}/${this.context.componentClassName}.js`,
                            manifest: `./src/${this.folderName}/${this.context.componentNameCamelCase}/${this.context.componentClassName}.manifest.json`
                        }
                    ]
                }, this.context.componentClassNameKebabCase);
            }
        } catch (error) {
            // @todo move this error to occur during prompting
            this.log(_colors.default.red(`Cannot create two ${this.friendlyName}s with the same name.`));
            throw error;
        }
        const localizedResourcePath = `lib/${this.folderName}/${this.context.componentNameCamelCase}/loc/{locale}.js`;
        this.configJson.addLocalizedResource(this.context.componentStrings, localizedResourcePath);
    }
    writing(shouldWrite = true) {
        super.writing(false);
        if (shouldWrite) {
            const dest = this._getOutputFolder(this.context.componentNameCamelCase);
            this.copyTemplate(_path.join(this.sourceRoot(), this.context.template), dest);
            this.copyTemplate(_path.join(this.sourceRoot(), 'base'), dest);
        }
        // we need to write the eslint config anyway
        this.copyTemplate(_path.join(`${__dirname}/templates`, 'eslint'), this.destinationRoot());
    }
    install() {
        if (!this.tryInstall()) {
            this.log();
            this.log(`Skipping installation of dependencies. You should run "npm install"`);
        }
    }
    end() {
        if (this._shouldLogESLintWarning) {
            this.log(_colors.default.yellow(`Found an existing ".eslintrc.js" file. Consider updating it if you are adding React component to non-React solution.`));
        }
    }
    _getOutputFolder(componentNameCamelCase) {
        return _path.join(this.destinationRoot(), 'src', this.folderName, componentNameCamelCase);
    }
    async _checkSolutionAsync() {
        if (this.config.get('creatingSolution')) {
            return;
        }
        const solutionName = await _utilities.checkSolutionFolder(this, _YeomanConfiguration.YeomanConfiguration.libraryName);
        this.log(`Add new ${this.friendlyName} to solution ${_colors.default.cyan(solutionName)}.`);
    }
    constructor(...args){
        super(...args);
        _define_property(this, "_shouldLogESLintWarning", false);
    }
}
function normalizeComponentNames(componentNameUnescaped, componentType) {
    //  title-case:   Hello, World!     =>  HelloWorld
    const componentName = _utilities.titleCase(componentNameUnescaped);
    //  camel-case:   HelloWorld        =>  helloWorld
    const componentNameCamelCase = _lodash.camelCase(componentName);
    //  class-name:   HelloWorld        => HelloWorldWebPart
    const componentClassName = componentName + componentType;
    //  strings:      HelloWorldWebPart => HelloWorldWebPartStrings
    const componentStrings = componentClassName + 'Strings';
    //  kebab-case:   HelloWorldWebPart =>  hello-world-web-part
    let componentClassNameKebabCase = _lodash.kebabCase(componentClassName);
    const MAX_BUNDLE_NAME_LENGTH = 64;
    if (componentClassNameKebabCase.length > MAX_BUNDLE_NAME_LENGTH) {
        componentClassNameKebabCase = componentClassNameKebabCase.substring(0, MAX_BUNDLE_NAME_LENGTH);
        //  a trailing - will also cause validation to fail
        const lastCharacterIndex = componentClassNameKebabCase.length - 1;
        if (componentClassNameKebabCase[lastCharacterIndex] === '-') {
            componentClassNameKebabCase = componentClassNameKebabCase.substring(0, lastCharacterIndex);
        }
    }
    // alias based off componentNameTitleCase, but truncate if it is too long
    let componentAlias = componentClassName;
    const MAX_ALIAS_LENGTH = 40;
    if (componentAlias.length > MAX_ALIAS_LENGTH) {
        componentAlias = componentAlias.substring(0, MAX_ALIAS_LENGTH);
    }
    return {
        componentNameUnescaped,
        componentName,
        componentNameCamelCase,
        componentClassName,
        componentStrings,
        componentClassNameKebabCase,
        componentAlias
    };
}

//#sourceMappingUrl=./BaseComponentGenerator.js.map