"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    ComponentSelectorGenerator: function() {
        return ComponentSelectorGenerator;
    },
    composeWith: function() {
        return composeWith;
    },
    defineOptions: function() {
        return defineOptions;
    }
});
const _colors = /*#__PURE__*/ _interop_require_default(require("colors"));
const _path = /*#__PURE__*/ _interop_require_wildcard(require("path"));
const _BaseGenerator = require("../../common/BaseGenerator");
const _utilities = /*#__PURE__*/ _interop_require_wildcard(require("../../common/utilities"));
const _webpart = /*#__PURE__*/ _interop_require_wildcard(require("../webpart"));
const _extension = /*#__PURE__*/ _interop_require_wildcard(require("../extension"));
const _library = /*#__PURE__*/ _interop_require_wildcard(require("../library"));
const _adaptiveCardExtension = /*#__PURE__*/ _interop_require_wildcard(require("../adaptiveCardExtension"));
function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interop_require_wildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {
        __proto__: null
    };
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
function composeWith(base, options) {
    _utilities.compose(_path.basename(__dirname), base, options);
}
function defineOptions(generator) {
    generator.option('component-type', {
        description: `The type of component. Currently "webpart", "extension", "library", or "adaptiveCardExtension"`,
        type: String
    });
    _webpart.defineOptions(generator);
    _extension.defineOptions(generator);
    _library.defineOptions(generator);
    _adaptiveCardExtension.defineOptions(generator);
}
class ComponentSelectorGenerator extends _BaseGenerator.BaseGenerator {
    initializing() {
    /* no-op */ }
    async prompting() {
        this.ensureCorrectFolder();
        if (!this._shouldExecute()) {
            return;
        }
        let choices;
        if (this.config.get('isDomainIsolated')) {
            choices = [
                {
                    name: 'WebPart',
                    value: 'webpart'
                }
            ];
        } else {
            choices = [
                {
                    name: 'WebPart',
                    value: 'webpart'
                },
                {
                    name: 'Extension',
                    value: 'extension'
                },
                {
                    name: 'Library',
                    value: 'library'
                },
                {
                    name: 'Adaptive Card Extension',
                    value: 'adaptiveCardExtension'
                }
            ];
        }
        const answers = await this.prompt([
            {
                type: 'list',
                name: 'componentType',
                default: 'webpart',
                when: ()=>!this.config.get('componentType') && !this.config.get('extensionType'),
                message: 'Which type of client-side component to create?',
                choices: choices
            }
        ]);
        const validComponentTypes = new Set(choices.map((choice)=>choice.value));
        const componentType = answers.componentType || this.config.get('componentType');
        if (validComponentTypes.has(componentType)) {
            this.config.set('componentType', componentType);
        } else {
            this.log(_colors.default.red(`Invalid componentType: "${componentType}"`));
            if (this.config.get('extensionType')) {
                this.log(_colors.default.red(`Invalid extensionType: "${this.config.get('extensionType')}"`));
            }
        }
    }
    configuring() {
    /* no-op */ }
    writing() {
    /* no-op */ }
    install() {
    /* no-op */ }
    end() {
    /* no-op */ }
    _shouldExecute() {
        return this.config.get('environment') === 'spo';
    }
    constructor(...args){
        super(...args);
        _define_property(this, "friendlyName", 'Component Selection');
    }
}
// Yeoman expects exports of this type, which overrides the TS exports above
// Ensure that anything marked as export above is also listed here
module.exports = ComponentSelectorGenerator;
module.exports.defineOptions = defineOptions;
module.exports.composeWith = composeWith;

//#sourceMappingUrl=./index.js.map