"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    BaseExtensionGenerator: function() {
        return BaseExtensionGenerator;
    },
    defineOptions: function() {
        return defineOptions;
    }
});
const _path = /*#__PURE__*/ _interop_require_wildcard(require("path"));
const _safe = /*#__PURE__*/ _interop_require_default(require("colors/safe"));
const _nodecorelibrary = require("@rushstack/node-core-library");
const _utilities = /*#__PURE__*/ _interop_require_wildcard(require("../../common/utilities"));
const _BaseComponentGenerator = /*#__PURE__*/ _interop_require_wildcard(require("../component/BaseComponentGenerator"));
function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interop_require_wildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {
        __proto__: null
    };
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
function defineOptions(generator, type = 'Extension') {
    _BaseComponentGenerator.defineOptions(generator, type);
}
class BaseExtensionGenerator extends _BaseComponentGenerator.BaseComponentGenerator {
    configuring() {
        super.configuring();
        this.ensureDependencyGroup('extension');
        this.addFeature();
    }
    writing(shouldCopy = true) {
        super.writing(shouldCopy);
        if (shouldCopy) {
            if (!this.hasElementsXml()) {
                const dest = _path.join(this.destinationRoot(), 'sharepoint');
                this.copyTemplate(_path.join(this.sourceRoot(), 'sharepoint'), dest);
            } else {
                const docsUrl = this._getFeatureFrameworkDocsUrl();
                if (docsUrl) {
                    this.log(_safe.default.yellow(`Elements.xml already exists. Please, reference ${docsUrl} to add additional elements to it.`));
                }
            }
        }
    }
    install() {
        super.install();
    }
    addFeature() {
        if (!this.hasElementsXml()) {
            this.packageSolutionJson.addExtensionFeature(_utilities.generateGuid(), this.includeClientSideInstances());
        }
    }
    hasElementsXml() {
        const elementsXmlPath = _path.join(this.destinationRoot(), 'sharepoint', 'assets', 'elements.xml');
        return _nodecorelibrary.FileSystem.exists(elementsXmlPath);
    }
    _getFeatureFrameworkDocsUrl() {
        const extensionType = this.config.get('extensionType');
        switch(extensionType){
            case 'FieldCustomizer':
                return 'https://aka.ms/spfx/extensions/field';
            case 'ListViewCommandSet':
                return 'https://aka.ms/spfx/extensions/cmdset';
            case 'ApplicationCustomizer':
                return 'https://aka.ms/spfx/extensions/tutorial/helloworld3';
            case 'SearchQueryModifier':
                return 'https://aka.ms/spfx/extensions/search';
            case 'FormCustomizer':
                return ''; // TODO: add form customizer docs url
            default:
                return '';
        }
    }
    constructor(...args){
        super(...args);
        _define_property(this, "folderName", 'extensions');
    }
}

//#sourceMappingUrl=./BaseExtensionGenerator.js.map