"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    ExtensionSelectorGenerator: function() {
        return ExtensionSelectorGenerator;
    },
    composeWith: function() {
        return composeWith;
    },
    defineOptions: function() {
        return defineOptions;
    }
});
const _colors = /*#__PURE__*/ _interop_require_default(require("colors"));
const _path = /*#__PURE__*/ _interop_require_wildcard(require("path"));
const _BaseGenerator = require("../../common/BaseGenerator");
const _utilities = /*#__PURE__*/ _interop_require_wildcard(require("../../common/utilities"));
const _commandSet = /*#__PURE__*/ _interop_require_wildcard(require("../commandSet"));
const _fieldCustomizer = /*#__PURE__*/ _interop_require_wildcard(require("../fieldCustomizer"));
const _formCustomizer = /*#__PURE__*/ _interop_require_wildcard(require("../formCustomizer"));
function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interop_require_wildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {
        __proto__: null
    };
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
function defineOptions(generator) {
    _fieldCustomizer.defineOptions(generator);
    _commandSet.defineOptions(generator);
    _formCustomizer.defineOptions(generator);
    generator.option('extension-type', {
        description: `The type of extension. Currently "ApplicationCustomizer", "FieldCustomizer", "ListViewCommandSet", "FormCustomizer", or "SearchQueryModifier"`,
        type: String
    });
}
function composeWith(base, options) {
    _utilities.compose(_path.basename(__dirname), base, options);
}
class ExtensionSelectorGenerator extends _BaseGenerator.BaseGenerator {
    initializing() {
    /* no-op */ }
    async prompting() {
        this.ensureCorrectFolder();
        if (!this._shouldExecute()) {
            return;
        }
        const choices = [
            {
                name: 'Application Customizer',
                value: 'ApplicationCustomizer'
            },
            {
                name: 'Field Customizer',
                value: 'FieldCustomizer'
            },
            {
                name: 'ListView Command Set',
                value: 'ListViewCommandSet'
            },
            {
                name: 'Form Customizer',
                value: 'FormCustomizer'
            },
            {
                name: 'Search Query Modifier',
                value: 'SearchQueryModifier'
            }
        ];
        const validExtensionTypes = new Set(choices.map((choice)=>choice.value));
        const answers = await this.prompt([
            {
                type: 'list',
                name: 'extensionType',
                default: 'ApplicationCustomizer',
                when: ()=>!this.config.get('extensionType'),
                message: 'Which type of client-side extension to create?',
                choices
            }
        ]);
        const extensionType = answers.extensionType || this.config.get('extensionType');
        if (validExtensionTypes.has(extensionType)) {
            this.config.set('extensionType', extensionType);
        } else {
            this.log(_colors.default.red(`Invalid extensionType: "${extensionType}"`));
        }
    }
    configuring() {
    /* no-op */ }
    writing() {
    /* no-op */ }
    install() {
    /* no-op */ }
    end() {
    /* no-op */ }
    _shouldExecute() {
        return this.config.get('environment') === 'spo' && this.config.get('componentType') === 'extension';
    }
    constructor(args, options){
        super(args, options);
        _define_property(this, "friendlyName", 'Extension Selection');
    }
}
// Yeoman expects exports of this type, which overrides the TS exports above
// Ensure that anything marked as export above is also listed here
module.exports = ExtensionSelectorGenerator;
module.exports.defineOptions = defineOptions;
module.exports.composeWith = composeWith;

//#sourceMappingUrl=./index.js.map