"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    composeWith: function() {
        return composeWith;
    },
    defineOptions: function() {
        return defineOptions;
    }
});
const _lodash = /*#__PURE__*/ _interop_require_wildcard(require("lodash"));
const _path = /*#__PURE__*/ _interop_require_wildcard(require("path"));
const _utilities = /*#__PURE__*/ _interop_require_wildcard(require("../../common/utilities"));
const _BaseGenerator = require("../../common/BaseGenerator");
const _YeomanConfiguration = require("../../common/YeomanConfiguration");
function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interop_require_wildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {
        __proto__: null
    };
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
function composeWith(base, options) {
    _utilities.compose(_path.basename(__dirname), base, options);
}
function defineOptions(generator) {
    generator.option('solution-name', {
        description: 'Solution name, as well as folder name',
        type: String
    });
    generator.option('framework', {
        description: 'Framework to use. Options are: "none", "react"',
        type: String
    });
    generator.option('environment', {
        description: 'The target environment for the solution. "spo".',
        type: String
    });
    generator.option('package-manager', {
        description: 'The package manager for the solution. Options are: "npm", "pnpm", or "yarn"',
        type: String
    });
    generator.option('skip-feature-deployment', {
        description: 'If specified, allow the tenant admin the choice of being able to ' + 'deploy the components to all sites immediately without running any ' + 'feature deployment or adding apps in sites',
        type: Boolean
    });
    generator.option('is-domain-isolated', {
        description: 'If specified, the solution will have permissions to access web APIs ' + 'that are unique not shared with other components in the tenant',
        type: Boolean
    });
    generator.option('use-heft', {
        description: 'If specified, the solution will build the project using Heft',
        type: Boolean
    });
}
class SolutionGenerator extends _BaseGenerator.BaseGenerator {
    initializing() {
    /* no-op */ }
    async prompting() {
        if (!this._shouldExecute()) {
            return;
        }
        const answers = await this.prompt([
            {
                type: 'input',
                name: 'solutionName',
                default: _lodash.kebabCase(this.appname),
                when: ()=>!this.config.get('solutionName'),
                message: 'What is your solution name?'
            }
        ]);
        const environment = 'spo';
        const solutionName = this.config.get('solutionName') || answers.solutionName;
        const solutionShortDescription = this.config.get('solutionShortDescription') || `${solutionName} description`;
        // allows to set --skip-feature-deployment false
        let skipFeatureDeployment = undefined;
        const skipFeatureDeploymentConfig = this.config.get('skipFeatureDeployment');
        if (skipFeatureDeploymentConfig !== undefined) {
            if (typeof skipFeatureDeploymentConfig === 'string') {
                skipFeatureDeployment = skipFeatureDeploymentConfig.toLowerCase() === 'true';
            } else {
                skipFeatureDeployment = skipFeatureDeploymentConfig;
            }
        } else {
            skipFeatureDeployment = true;
        }
        const componentType = this.config.get('componentType');
        const extensionType = this.config.get('extensionType');
        const template = this.config.get('framework') || this.config.get('template');
        const componentName = this.config.get('componentName');
        const componentDescription = this.config.get('componentDescription');
        const isCreatingSolution = this.config.get('isCreatingSolution');
        // allows to set --is-domain-isolated false
        let isDomainIsolated = undefined;
        const isDomainIsolatedConfig = this.config.get('isDomainIsolated');
        if (isDomainIsolatedConfig !== undefined) {
            if (typeof isDomainIsolatedConfig === 'string') {
                isDomainIsolated = isDomainIsolatedConfig.toLowerCase() === 'true';
            } else {
                isDomainIsolated = isDomainIsolatedConfig;
            }
        } else {
            isDomainIsolated = false;
        }
        // allows to set --use-heft false
        let useHeft = answers.useHeft;
        const useHeftConfig = this.config.get('useHeft');
        if (useHeftConfig !== undefined) {
            if (typeof useHeftConfig === 'string') {
                useHeft = useHeftConfig.toLowerCase() === 'true';
            } else {
                useHeft = useHeftConfig;
            }
        }
        const isPlusBeta = this.config.get('plusBeta');
        this.context.solutionName = solutionName;
        this.context.useHeft = !!useHeft;
        this.context.libraryName = _lodash.kebabCase(this.context.solutionName);
        this.context.libraryId = _utilities.generateGuid();
        this.context.solutionShortDescription = solutionShortDescription;
        this.context.skipFeatureDeployment = skipFeatureDeployment;
        this.context.isDomainIsolated = isDomainIsolated;
        this.context.version = _BaseGenerator.BaseGenerator.generatorPackageJson.version;
        this.context.versionBadge = `https://img.shields.io/badge/version-${this.context.version.replace('-', '--')}-${this.context.version.indexOf('-') !== -1 ? 'yellow' : 'green'}.svg`;
        // store some variables
        _YeomanConfiguration.YeomanConfiguration.version = _BaseGenerator.BaseGenerator.generatorPackageJson.version;
        _YeomanConfiguration.YeomanConfiguration.libraryName = this.context.libraryName;
        _YeomanConfiguration.YeomanConfiguration.libraryId = this.context.libraryId;
        _YeomanConfiguration.YeomanConfiguration.environment = environment;
        _YeomanConfiguration.YeomanConfiguration.packageManager = this.config.get('packageManager') || _YeomanConfiguration.YeomanConfiguration.packageManager || 'npm';
        _YeomanConfiguration.YeomanConfiguration.useHeft = !!useHeft;
        this.config.set('solutionName', solutionName);
        this.config.set('useHeft', useHeft);
        this.config.set('environment', environment);
        this.config.set('solutionShortDescription', solutionShortDescription);
        this.config.set('skipFeatureDeployment', skipFeatureDeployment);
        this.config.set('componentType', componentType);
        this.config.set('extensionType', extensionType);
        this.config.set('template', template);
        this.config.set('componentName', componentName);
        this.config.set('componentDescription', componentDescription);
        this.config.set('isCreatingSolution', isCreatingSolution);
        this.config.set('isDomainIsolated', isDomainIsolated);
        this.config.set('plusBeta', isPlusBeta);
        this.packageSolutionJson.setUpSolution(this.context);
    }
    configuring() {
        if (this._shouldExecute()) {
            const packageJsonTemplate = this.context.useHeft ? require('./heft.package.json') : require('./non-heft.package.json');
            if (this.packageJsonManager.data) {
                this.log(`Overwriting an existing "package.json"`);
            }
            packageJsonTemplate.name = this.context.libraryName;
            this.packageJsonManager.set(packageJsonTemplate);
            if (this.context.useHeft) {
                this.ensureDependencyGroup('useHeft');
            } else {
                this.ensureDependencyGroup('useGulp');
            }
            if (this.configJson.data) {
                this.log(`Overwriting an existing "config.json"`);
            }
            this.configJson.set({
                $schema: 'https://developer.microsoft.com/json-schemas/spfx-build/config.2.0.schema.json',
                version: '2.0',
                bundles: {},
                externals: {},
                localizedResources: {}
            });
        }
    }
    writing() {
        if (this._shouldExecute()) {
            // eslint-disable-next-line no-console
            console.log();
            super.writing(false);
            const dest = this.destinationRoot();
            this.copyTemplate(_path.join(this.sourceRoot(), 'base'), dest);
            if (this.context.useHeft) {
                this.copyTemplate(_path.join(this.sourceRoot(), 'heft'), dest);
            } else {
                this.copyTemplate(_path.join(this.sourceRoot(), 'gulp'), dest);
                this.copyTemplate(_path.join(this.sourceRoot(), _YeomanConfiguration.YeomanConfiguration.environment), dest);
            }
            this.packageJsonManager.save();
            this.configJson.save();
            this.packageSolutionJson.save();
        }
    }
    install() {
        if (this._shouldExecute()) {
            // eslint-disable-next-line no-console
            console.log();
            this.tryInstall();
        }
    }
    end() {
    /* no-op */ }
    _shouldExecute() {
        return this.config.get('isCreatingSolution');
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    constructor(args, options){
        super(args, options);
        _define_property(this, "allowEmptyPackageJson", true);
        _define_property(this, "friendlyName", 'Solution');
        defineOptions(this);
    }
}
// Yeoman expects exports of this type, which overrides the TS exports above
// Ensure that anything marked as export above is also listed here
module.exports = SolutionGenerator;
module.exports.composeWith = composeWith;
module.exports.defineOptions = defineOptions;

//#sourceMappingUrl=./index.js.map